﻿Shader "Custom/NewCutShader"
{
    Properties
    {
        _Color ("Color", Color) = (1,1,1,1)
        _MainTex ("Albedo (RGB)", 2D) = "white" {}
        _Glossiness ("Smoothness", Range(0,1)) = 0.5
        _Metallic ("Metallic", Range(0,1)) = 0.0
    }
    SubShader
    {
        //Cull Off
        Tags { "RenderType"="Opaque" }
        LOD 200

        CGPROGRAM
        // Physically based Standard lighting model, and enable shadows on all light types
        #pragma surface surf Standard fullforwardshadows

        // Use shader model 3.0 target, to get nicer looking lighting
        #pragma target 3.0

        sampler2D _MainTex;

        // custom variables here
        uniform float4 _MyBoxWorldPos;
        uniform float _MyConeAngle;

        struct Input
        {
            float2 uv_MainTex;
            float3 worldPos;
            float3 worldNormal; INTERNAL_DATA
            float facing : VFACE;
        };

        half _Glossiness;
        half _Metallic;
        fixed4 _Color;

        // Add instancing support for this shader. You need to check 'Enable Instancing' on materials that use the shader.
        // See https://docs.unity3d.com/Manual/GPUInstancing.html for more information about instancing.
        // #pragma instancing_options assumeuniformscaling
        UNITY_INSTANCING_BUFFER_START(Props)
        // put more per-instance properties here
        UNITY_INSTANCING_BUFFER_END(Props)

        void surf (Input IN, inout SurfaceOutputStandard o)
        {
            // _MyBoxWorldPos.xyz is box's center point
            float3 conePole = _WorldSpaceCameraPos.xyz - _MyBoxWorldPos.xyz;
            float3 coneDir = normalize(conePole);
            
            // _MyConeAngle is in degrees
            float coneSin, coneCos;
            sincos(_MyConeAngle * UNITY_PI / 180, coneSin, coneCos);
            
            // _MyBoxWorldPos.w is approximately the box's spherical bounds radius
            // The radius is divided by coneSin so that the radius of the cone is the radius of the box's bounds
            // by placing the cone's tip back further
            float3 surfacePosToConeDir = normalize(IN.worldPos - _MyBoxWorldPos.xyz + coneDir * _MyBoxWorldPos.w / coneSin);
            
            // get the current surface position's cosine angle
            float surfacePosAngle = dot(coneDir, surfacePosToConeDir);
            
            // get distance from box along cone's pole
            float distFromBox = dot(coneDir, IN.worldPos - _MyBoxWorldPos.xyz);
            
            // clip capped cone
            if (_MyBoxWorldPos.w > 0) {
                clip(-min(step(coneCos, surfacePosAngle), distFromBox));
            }

            //o.Normal = WorldToTangentNormalVector(IN, IN.worldNormal);

            fixed4 c = tex2D (_MainTex, IN.uv_MainTex) * _Color;
            o.Albedo = c.rgb;

            // Metallic and smoothness come from slider variables
            o.Metallic = _Metallic;
            o.Smoothness = _Glossiness;
            o.Alpha = c.a;            
        }
        ENDCG
    }
    FallBack "Diffuse"
}
